package cn.kdan.cloud.pdf.office.payment.service;

import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author ComPDFKit-WPH 2023-01-29
 */
public interface OrderService extends IService<Order> {

    /**
     * 创建订单
     *
     * @param creatOrderDTO creatOrderDTO
     * @return CreatOrderResultVO
     */
    CreatOrderResultVO creatOrder(CreatOrderDTO creatOrderDTO);


    /**
     * 订单创建（applePay专用）
     *
     * @param creatOrderDTO creatOrderDTO
     * @return CreatOrderResultVO
     */
    @Transactional
    CreatOrderResultVO appStoreCreateOrder(CreatOrderDTO creatOrderDTO);

    /**
     * 关闭订单
     *
     * @param orderId orderId
     */
    void closeOrder(String orderId);

    /**
     * 订单支付成功
     *
     * @param orderSucceeded orderSucceeded
     * @return 订单id
     */
    OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    OrdersVO getOrderById(String orderId);

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    List<OrdersVO> getOrderByTradeNo(String tradeNo);

    /**
     * 手动创建虚拟订单
     *
     * @param creatOrderManual creatOrderManual
     * @return OrderUpdateVO
     */
    OrderUpdateVO creatOrderManual(CreatOrderManualDTO creatOrderManual);

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     */
    void chargebackOrder(ChargebackOrderDTO chargebackOrderDTO);

    /**
     * 新增order
     *
     * @param order order
     */
    void insertOrder(Order order);

    /**
     * 获取订单
     *
     * @param userId userId
     * @param appId appId
     * @return OrdersVO
     */
    List<OrdersVO> getOrderByUserId(String userId,String appId);

    /**
     * 订单支付成功（applePay专用）
     * 回传appal pay id &签名证书
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     */
    void appStoreOrderSucceed(AppStoreOrderSucceedDTO appStoreOrderSucceed);

    /**
     * 获取订单
     *
     * @param thirdOrderNo thirdOrderNo
     * @return OrdersVO
     */
    OrdersVO getOrderByThirdOrderNo(String thirdOrderNo);
}
