package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.bo.*;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.UserAction;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.product.enums.ProductConstant;
import cn.kdan.cloud.pdf.office.api.payment.enums.ProductTypeEnum;
import cn.kdan.cloud.pdf.office.api.payment.paypal.ApplicationContext;
import cn.kdan.cloud.pdf.office.api.payment.paypal.Money;
import cn.kdan.cloud.pdf.office.api.payment.paypal.PayerName;
import cn.kdan.cloud.pdf.office.api.payment.paypal.SubscriberRequest;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.constant.RedisKeyConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.OrderMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.cache.RedisCache;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-29
 * <p>
 * OrdersServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    private final ProductApi productApi;

    private final UserApi userApi;

    private final PaddleClient paddleClient;

    private final RabbitTemplate rabbitTemplate;


    private final PaypalClient paypalClient;

     @Resource(name = "myRedisCacheManager")
    private  RedisCacheManager myRedisCacheManager;
//    private final SubscriptionsService subscriptionsService;

    @Override
    @Transactional
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#1#h",
            key = "T(String).valueOf(#creatOrderDTO.email)" +
                    ".concat('-').concat(#creatOrderDTO.productId)" +
                    ".concat('-').concat(#creatOrderDTO.paymentMethod.getValue())",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public CreatOrderResultVO creatOrder(CreatOrderDTO creatOrderDTO) {
        //判断是否有首年优惠
        ProductVO product = productApi.getProductForPay(creatOrderDTO.getProductId(),creatOrderDTO.getPaymentMethod().getValue(),creatOrderDTO.getIsDiscount()).getResult();
        PaymentMethodEnum paymentMethodEnum = creatOrderDTO.getPaymentMethod();
        log.info("creatOrder:{}", creatOrderDTO);
        Order order = new Order();
        order.setAppId(product.getAppId().toString());
        if(StringUtils.isNotEmpty(creatOrderDTO.getUserId())){
            order.setUserId(creatOrderDTO.getUserId());
        }
        order.setEmail(creatOrderDTO.getEmail());
        order.setProductId(creatOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
        String readeNo = MyDateUtils.getTimeStamp() + "-" + creatOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        switch (paymentMethodEnum) {
            case WXPAY:
                break;
            case ALIPAY:
                break;
            case PADDLE:
                //设置折扣价格
                BigDecimal price = creatOrderDTO.getIsDiscount() == 0 ? product.getPrice():product.getDisplayPrice();
                GeneratePayLinkPaddleBO paddleBO = new GeneratePayLinkPaddleBO();
                paddleBO.setPrices(new String[]{"USD:" + price.toString()});
                paddleBO.setProductId(product.getThirdPayId());
                paddleBO.setPassthrough(readeNo);
                GeneratePayLinkResultPaddleBO resultPaddleBO = paddleClient.creatSubscriptions(paddleBO);
                order.setPagePayUrl(resultPaddleBO.getUrl());
                order.setResult(paddleBO.toString());
                order.setThirdOrderNo(paddleBO.getPassthrough());
                //订单类型1网站购买
                order.setType(1);
                //设置第三方订单
                order.setThirdTradeNo(paddleBO.getPassthrough());
                //设置支付方式
                order.setPayment(paymentMethodEnum.getValue());
                break;
            case NO_NEED_TO_PAY:
                break;
            case PAYPAL:
                //创建paypal订阅
                if(ProductTypeEnum.PLAN.getValue().equals(product.getType())){
                    CreatSubscriptionsPaypalBO creatSubscriptionsBO = new CreatSubscriptionsPaypalBO();
                    ApplicationContext applicationContext = new ApplicationContext();
                    applicationContext.setBrand_name("KDanMobile");
                    applicationContext.setLocale("zh-CN");
                    applicationContext.setUser_action(UserAction.SUBSCRIBE_NOW);
                    //成功或失败回调url
                    applicationContext.setReturn_url("https://www.baidu.com");
                    applicationContext.setCancel_url("https://www.bilibili.com");
                    //订阅计划id
                    creatSubscriptionsBO.setPlan_id(product.getThirdPayId());
                    creatSubscriptionsBO.setApplication_context(applicationContext);
                    CreatSubscriptionsResultPaypalBO resultBO = paypalClient.creatSubscriptions(creatSubscriptionsBO);
                    order.setResult(resultBO.toString());
                    order.setThirdOrderNo(resultBO.getId());
                    //订单类型1网站购买
                    order.setType(1);
                    //设置第三方订单
                    order.setThirdTradeNo(resultBO.getId());
                    //设置支付方式
                    order.setPayment(paymentMethodEnum.getValue());
                    order.setPagePayUrl(resultBO.getLinks().get(0).getHref());
                };
                break;
            // 当前支付方式无效
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
        this.baseMapper.insert(order);
        CreatOrderResultVO creatOrderResultVO = new CreatOrderResultVO();
        BeanUtils.copyProperties(order, creatOrderResultVO);
        // 订单关闭
        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                order.getId(),
                (msg -> {
                    msg.getMessageProperties().setHeader("x-delay", ( 3 * 60 * 60 * 1000) + "");
                    return msg;
                }));
        return creatOrderResultVO;
    }

    @Override
    @Transactional
    public CreatOrderResultVO appStoreCreateOrder(CreatOrderDTO creatOrderDTO) {
        PaymentMethodEnum paymentMethodEnum = creatOrderDTO.getPaymentMethod();
        log.info("creatOrder:{}", creatOrderDTO);
        Order order = new Order();
        order.setAppId(creatOrderDTO.getAppId());
        order.setUserId(creatOrderDTO.getUserId());
        order.setProductId(creatOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
//        orders.setDiscount(creatOrderDTO.getDiscount());
        String readeNo = MyDateUtils.getTimeStamp() + "-" + creatOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        //订单类型1网站购买
        order.setType(1);
        this.baseMapper.insert(order);
        CreatOrderResultVO creatOrderResultVO = new CreatOrderResultVO();
        BeanUtils.copyProperties(order, creatOrderResultVO);
//        // 订单关闭
//        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
//        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
//                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
//                order.getId(),
//                (msg -> {
//                    msg.getMessageProperties().setHeader("x-delay", ( 3 * 60 * 60 * 1000) + "");
//                    return msg;
//                }));
        return creatOrderResultVO;
    }

    @Override
    public void closeOrder(String orderId) {
        OrdersVO orders = getOrderById(orderId);
        log.info("当前订单信息：{}", orders);
        if (ObjectUtils.isNotEmpty(orders.getStatus())&&orders.getStatus().equals(OrderConstant.CREATED)) {
            Order order = new Order();
            order.setStatus(OrderConstant.CLOSED);
            order.setId(orderId);
            this.baseMapper.updateById(order);
        }
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orders.getEmail() + "-" + orders.getProductId() + "-" + orders.getPayment());
    }

    @Override
    public OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded) {
        Order orders = new Order();
        orders.setThirdOrderNo(orderSucceeded.getThirdOrderNo());
        orders.setThirdTradeNo(orderSucceeded.getThirdTradeNo());
        orders.setPrice(orderSucceeded.getPrice());
        orders.setUserId(orderSucceeded.getUserId());
        orders.setPayDate(new Date());
        orders.setSubscriptionId(orderSucceeded.getSubscriptionId());
        orders.setStatus(OrderConstant.COMPLETED);
        this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
        Order orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                .select(Order::getId, Order::getUserId,Order::getProductId,Order::getAppId,Order::getPayment,Order::getEmail)
                .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orderTradeNo.getEmail() + "-" + orderTradeNo.getProductId() + "-" + orderTradeNo.getPayment());
        return OrderUpdateVO.builder()
                .orderId(orderTradeNo.getId())
                .userId(orderTradeNo.getUserId())
                .productId(orderTradeNo.getProductId())
                .appId(orderTradeNo.getAppId())
                .build();
    }

    /**
     * 删除redis缓存管理器中的对应key的缓存
     * @param key key
     */
    private void deleteTheCacheInTheCacheManager(String key) {
        log.info("删除缓存中的订单 key"+ key);
        RedisCache cache = (RedisCache) myRedisCacheManager.getCache(RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER);
        if (cache != null) {
            cache.evictIfPresent(key);
        }
    }

    @Override
    public OrdersVO getOrderById(String orderId) {
        OrdersVO ordersVO = new OrdersVO();
        Order orders = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId));
        if(ObjectUtils.isEmpty(orders)){
            return ordersVO;
        }
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId)), ordersVO);
        return ordersVO;
    }

    @Override
    public List<OrdersVO> getOrderByTradeNo(String tradeNo) {
        return this.baseMapper.selectOrdersByTradeNo(tradeNo);
    }


    @Override
    public OrderUpdateVO creatOrderManual(CreatOrderManualDTO creatOrderManual) {
        Order orders = new Order();
        orders.setAppId(creatOrderManual.getAppId());
        orders.setProductId(creatOrderManual.getProductId());
        orders.setUserId(creatOrderManual.getUserId());
        orders.setTradeNo(creatOrderManual.getTradeNo());
        orders.setThirdTradeNo(creatOrderManual.getThirdTradeNo());
        if(StringUtils.isEmpty(creatOrderManual.getThirdOrderNo())){
            orders.setThirdOrderNo(creatOrderManual.getThirdOrderNo());
        }
        orders.setSubscriptionId(creatOrderManual.getSubscriptionId());
        orders.setPrice(creatOrderManual.getPrice());
        orders.setPayment(creatOrderManual.getPaymentMethod().getValue());
        orders.setStatus(OrderConstant.COMPLETED);
        orders.setPayDate(new Date());
        this.baseMapper.insert(orders);
        return OrderUpdateVO.builder()
                .userId(creatOrderManual.getUserId())
                .orderId(orders.getId())
                .build();
    }


    @Override
    public void chargebackOrder(@NotNull ChargebackOrderDTO chargebackOrderDTO) {
        log.info("手动退单修改状态：{}", chargebackOrderDTO);
        // 修改订单状态
        Order orders = new Order();
        orders.setId(chargebackOrderDTO.getOrderId());
        orders.setStatus(OrderConstant.REFUND);
        this.baseMapper.updateById(orders);
        log.info("手动退单修改状态成功：{}", chargebackOrderDTO);
    }

    @Override
    public void insertOrder(Order order) {
        this.baseMapper.insert(order);
    }

    @Override
    public List<OrdersVO> getOrderByUserId(String userId,String appId) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Order>()
                .eq(Order::getUserId,userId).eq(Order::getAppId, appId)).stream().map(order -> {
            OrdersVO ordersVO = new OrdersVO();
            BeanUtils.copyProperties(order,ordersVO);
            return ordersVO;
        }).collect(Collectors.toList());
    }

    @Override
    public void appStoreOrderSucceed(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        // 插入第三方订单id和票据信息

        // 验证票据信息
        // 修改订单状态
        // TODO appal pay不作为补救策略
    }

    @Override
    public OrdersVO getOrderByThirdOrderNo(String thirdOrderNo) {
        OrdersVO ordersVO = new OrdersVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getThirdOrderNo,thirdOrderNo)),ordersVO);
        return ordersVO;
    }

    /**
     * 对价格进行处理
     *
     * @param creatOrderDTO creatOrderDTO
     * @param price         price
     * @param orders        orders
     * @return 新价格
     */
    private BigDecimal priceHandle(CreatOrderDTO creatOrderDTO, BigDecimal price, Order orders) {
//        if (!ObjectUtils.isEmpty(creatOrderDTO.getFinalPrice())) {
//            price = creatOrderDTO.getFinalPrice();
//        }
        if (!ObjectUtils.isEmpty(creatOrderDTO.getCouponCode())) {
            String couponCode = creatOrderDTO.getCouponCode();
            // TODO 优惠价查询价格
            BigDecimal couponCodePrice = BigDecimal.ONE;
            price = price.subtract(couponCodePrice);
            orders.setReducedPrice(couponCodePrice);
        }
        return price;
    }


}
