package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.dto.UpdateUserForOrderDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CancelSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.*;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PaidTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.entity.Subscription;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.SubscriptionsMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class SubscriptionsServiceImpl extends ServiceImpl<SubscriptionsMapper, Subscription> implements SubscriptionsService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final PaypalClient paypalClient;

    private final UserApi userApi;

    private final OrderService ordersService;

    private final RabbitTemplate rabbitTemplate;

    @Override
    public IPage<Subscription> page(Subscription query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }


    @Override
    public String creatSubscription(CreatSubscriptionDTO subscriptionInfo) {
        log.info("creatSubscription Info ：{}", subscriptionInfo);
        /* 获取产品的订阅周期时长 */
        // 当前时间
        LocalDateTime now = LocalDateTime.now();
        // 初始时间
        LocalDateTime originalTime = getOriginalTime(subscriptionInfo.getUserId(),subscriptionInfo.getProductId(), now);
        // 通过计算获取结束时间
        LocalDateTime endTime = getEndTime(subscriptionInfo.getProductId(), originalTime,1);
        // 产生一条最新的订阅信息
        Subscription subscription = new Subscription();
        if(StringUtils.isEmpty(subscriptionInfo.getId())){
            subscriptionInfo.setId(CommonUtils.generateId());
        }
        subscription.setId(subscriptionInfo.getId());
        subscription.setProductId(subscriptionInfo.getProductId());
        subscription.setStatus(SubscriptionConstant.actived);
        subscription.setUserId(subscriptionInfo.getUserId());
        subscription.setStartDate(MyDateUtils.localDateTimeToDate(originalTime));
        subscription.setEndDate(MyDateUtils.localDateTimeToDate(endTime));
        subscription.setThirdSubscriptionId(subscriptionInfo.getThirdSubscriptionId());
        subscription.setOrderId(subscriptionInfo.getOrderId());
        subscription.setAppId(subscriptionInfo.getAppId());
        subscription.setPayment(subscriptionInfo.getPayment());
        subscription.setEmail(subscriptionInfo.getEmail());
        subscription.setSubscriptionType(subscriptionInfo.getSubscriptionType());
        //2官网购买 3后台赠送
        subscription.setBuyerType(2);
        this.baseMapper.insert(subscription);
        updateUserSubscriptionStatus(subscription);
        // 发送消息
        SubscriptionUtil.subscriptionExpiredSendNews(rabbitTemplate, subscription.getId(), endTime);
        // todo 更新userSubscription
        return subscription.getId();
    }


    private void updateUserSubscriptionStatus(Subscription subscription) {
        // 根据产品id查询产品信息
        ProductVO productVO = productApi.getProduct(subscription.getProductId()).getResult();
        // 获取产品平台
        Integer platform = productVO.getPlatform();
        UpdateUserForOrderDTO successDTO = new UpdateUserForOrderDTO();
        successDTO.setUserId(subscription.getUserId());
        successDTO.setPayTpe(PaidTypeEnum.SUBSCRIBE.value());
        successDTO.setPlatform(platform);
        successDTO.setUserStatus(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
        successDTO.setEndDate(subscription.getEndDate());
        successDTO.setAppId(subscription.getAppId());
        successDTO.setProductId(subscription.getProductId());
        userApi.updateUserForOrder(successDTO);
    }

    @Override
    public SubscriptionsVO getSubscriptions(String userId, PlatformEnum platform) {
        return this.baseMapper.selectNewestSubscription(userId, platform.getValue());
    }

    @Override
    public SubscriptionsVO getSubscriptionsByProductId(String userId, String productId) {
        return this.baseMapper.selectNewestSubscriptionByProductId(userId, productId);
    }

    @Override
    public String creatSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual) {
        CreatSubscriptionDTO creatSubscriptionDTO = new CreatSubscriptionDTO();
        BeanUtils.copyProperties(subscriptionManual, creatSubscriptionDTO);
        return this.creatSubscription(creatSubscriptionDTO);
    }

    @Override
    public String giveSubscriptionManual(CreatSubscriptionManualDTO subscriptionManual) {
        String subscriptionId = this.creatSubscriptionManual(subscriptionManual);
        // 获取当前产品的所属平台
        Integer platform = productApi.getProduct(subscriptionManual.getProductId()).getResult().getPlatform();
        // TODO 判断当前用户的当前平台是否已在订阅中
        // 修改用户状态
        // 修改用户状态为试用中
        UpdateUserForOrderDTO successDTO = new UpdateUserForOrderDTO();
        successDTO.setUserId(subscriptionManual.getUserId());
        successDTO.setPayTpe(PaidTypeEnum.ON_TRIAL.value());
        successDTO.setUserStatus(PDFOfficeUserSubscriptionStatusEnum.TRIAL.value());
        successDTO.setPlatform(platform);
        userApi.updateUserForOrder(successDTO);
        return subscriptionId;
    }

    @Override
    public void chargebackSubscription(ChargebackOrderDTO chargebackOrderDTO) {
        CancelSubscriptionDTO cancelSubscriptionDTO = new CancelSubscriptionDTO();
        cancelSubscriptionDTO.setSubscriptionId(chargebackOrderDTO.getSubscriptionId());
        this.chargebackSubscription(cancelSubscriptionDTO);
    }

    @Override
    public void chargebackSubscription(CancelSubscriptionDTO cancelSubscriptionDTO) {
        Subscription subscription = this.baseMapper.selectById(cancelSubscriptionDTO.getSubscriptionId());
        String thirdSubscriptionId = subscription.getThirdSubscriptionId();
        OrdersVO order = ordersService.getOrderById(subscription.getOrderId());
        switch (PaymentMethodEnum.fromValue(order.getPayment())){
            case PAYPAL:
                // 取消续订 webhook实际处理状态字段
                paypalClient.cancelSubscription(thirdSubscriptionId,"Not satisfied with the service");
                break;
            case PADDLE:
                // 更新paddle的订阅信息
                paddleClient.cancelSubscription(thirdSubscriptionId);
                break;
            case ALIPAY: break;
            case WXPAY: break;
            case NO_NEED_TO_PAY: // TODO 是否需要根据别的字段取消续订？看后续后台规划
                break;
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
    }

    @Override
    public SubscriptionSchemeVO getSubscriptionSchemeInfo(String userId,String appId) {
        List<SubscriptionsVO> subscriptions = this.baseMapper.selectByUserId(userId,appId);
        SubscriptionSchemeVO subscriptionSchemeVO = new SubscriptionSchemeVO();
        subscriptionSchemeVO.setSubscriptionInfo(getSubscriptionInfo(subscriptions));
        subscriptionSchemeVO.setOrdersInfo(getOrderInfo(userId,appId));
        return subscriptionSchemeVO;
    }

    @Override
    public SubscriptionsVO getSubscriptionBySubscriptionId(String subscriptionId) {
        return this.baseMapper.selectBySubscriptionId(subscriptionId);
    }

    /**
     * 修改订阅状态
     *
     * @param subscriptionId id
     * @param subscriptionStatus SubscriptionConstant
     */
    @Override
    public void updateSubscriptionStatus(String subscriptionId, Integer subscriptionStatus){
        // 修改订阅状态
        Subscription subscriptions = new Subscription();
        subscriptions.setId(subscriptionId);
        subscriptions.setStatus(subscriptionStatus);
        this.baseMapper.updateById(subscriptions);
    }

    @Override
    public List<SubscriptionsVO> getSubscriptionsByUserIdAndProductId(String userId, String productId) {
        return this.baseMapper.selectList(new LambdaQueryWrapper<Subscription>().eq(Subscription::getUserId,userId)
                .eq(Subscription::getProductId,productId)).stream().map(subscription -> {
            SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
            BeanUtils.copyProperties(subscription,subscriptionsVO);
            return subscriptionsVO;
        }).collect(Collectors.toList());
    }

    @Override
    public SubscriptionsVO getSubscriptionByThirdSubscriptionId(String thirdSubscriptionId) {
        SubscriptionsVO subscriptionsVO = new SubscriptionsVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Subscription>().eq(Subscription::getThirdSubscriptionId,thirdSubscriptionId)
                .orderByDesc(Subscription::getCreatedAt).last("Limit 1"))
                ,subscriptionsVO);
        return subscriptionsVO;
    }

    /**
     * 获取订阅结束时间
     *
     * @param productId productId
     * @param originalTime     开始时间
     * @param payTimes 第几次付款
     * @return 结束时间
     */
    private LocalDateTime getEndTime(String productId, LocalDateTime originalTime, Integer payTimes) {
        ProductVO product = productApi.getProduct(productId).getResult();
        //7.days:0,1.years:20,1.years:40
        if(product.getForPeriod().contains(",")){
            List<String> dateAndPriceList = Arrays.asList(product.getForPeriod().split(","));
            int times =  dateAndPriceList.size() > payTimes ? payTimes : dateAndPriceList.size()-1;
            String forPeriod = dateAndPriceList.get(times);;
            return MyDateUtils.TimeAddition(originalTime, forPeriod);
        }
        String forPeriod = product.getForPeriod();
        return MyDateUtils.TimeAddition(originalTime, forPeriod);
    }

    /**
     * 获取初始开始时间
     *
     * @param userId userId
     * @param now              now
     * @return LocalDateTime
     */
    private LocalDateTime getOriginalTime(String userId,String productId, LocalDateTime now) {
        // 通过用户id和产品id获取最新订阅信息
        SubscriptionsVO newestSubscriptions = getSubscriptionsByProductId(userId, productId);
        LocalDateTime originalTime;
        // 没有订阅数据
        if (ObjectUtils.isEmpty(newestSubscriptions)) {
            originalTime = now;
        } else {
            LocalDateTime localDateTime = MyDateUtils.dateToLocalDateTime(newestSubscriptions.getEndDate());
            // 当前时间大于最后结束时间
            if (now.compareTo(localDateTime) > 0) {
                originalTime = now;
            } else {
                originalTime = localDateTime;
            }
        }
        return originalTime;
    }

    private List<SubscriptionInfoVO> getSubscriptionInfo(List<SubscriptionsVO> subscriptions) {
        return subscriptions.stream().map(subscription -> {
            ProductVO productVO = productApi.getProduct(subscription.getProductId()).getResult();
            boolean isYearSubscription = productVO.getForPeriod().contains("years");
            return SubscriptionInfoVO.builder()
                    .price(subscription.getPrice())
                    .endDate(MyDateUtils.dateToLocalDateTime(subscription.getEndDate()))
                    .paymentMethod(subscription.getPayment())
                    .productName(productVO.getProductName())
                    .subscriptionPlan(isYearSubscription ? 2 : 1)
                    .productId(subscription.getProductId())
                    .build();
        }).collect(Collectors.toList());
    }

    private List<OrderInfoVO> getOrderInfo(String userId,String appId) {
        List<OrdersVO> ordersVOS = ordersService.getOrderByUserId(userId,appId);
        return ordersVOS.stream().map(ordersVO -> {
            ProductVO productVO = productApi.getProduct(ordersVO.getProductId()).getResult();
            boolean isYearSubscription = productVO.getForPeriod().contains("years");
            return OrderInfoVO.builder()
                    .date(ordersVO.getCreatedAt())
                    .price(ordersVO.getPrice())
                    .status(ordersVO.getStatus())
                    .subscriptionPlan(isYearSubscription ? 2 : 1)
                    .paymentMethod(ordersVO.getPayment())
                    .productId(ordersVO.getProductId())
                    .build();
        }).collect(Collectors.toList());
    }
}
