package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.api.email.feign.EmailApi;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.constant.SubscriptionConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionPayTypeEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PlatformTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import cn.kdan.cloud.pdf.office.payment.service.WebhookService;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class WebhookServiceImpl implements WebhookService {

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final UserApi userApi;

    private final UserSubscriptionInfoApi userSubscriptionInfoApi;
    private final ProductApi productApi;


    private final EmailApi emailApi;

    @Override
    public void handleSubsequentAutomaticDeduction(BigDecimal price, String thirdTradeNo, String thirdOrderId, OrdersVO orderByTradeNo) {
        log.info("自动订阅业务处理开始（非首期） paypal订单id：{}", thirdOrderId);
        // 如果不是第一期的话
        // 则直接增加订单，并且增加订阅时间
        // 判断当前是否存在订阅
        SubscriptionsVO subscriptions = subscriptionsService.getSubscriptionsByProductId(orderByTradeNo.getUserId(), orderByTradeNo.getProductId());
        // 根据第三方订阅id查询订阅表中的订阅
        if (ObjectUtils.isEmpty(subscriptions)) {
            throw new BackendRuntimeException(ErrorMessage.SUBSCRIPTION_NON_EXISTENT);
        }
        if (LocalDateTime.now().isAfter(MyDateUtils.dateToLocalDateTime(subscriptions.getEndDate()))) {
            throw new BackendRuntimeException(ErrorMessage.SUBSCRIPTION_NON_EXISTENT);
        }
        // 创建新订单
        CreatOrderManualDTO creatOrderManual = new CreatOrderManualDTO();
        BeanUtils.copyProperties(orderByTradeNo, creatOrderManual);
        creatOrderManual.setPaymentMethod(PaymentMethodEnum.fromValue(orderByTradeNo.getPayment()));
        // 设置实际扣费价格
        creatOrderManual.setPrice(price);
        // 设置第三方passthrough
        creatOrderManual.setThirdTradeNo(orderByTradeNo.getThirdTradeNo());
        // 设置第三方回调id
        creatOrderManual.setThirdOrderNo(thirdOrderId);
        creatOrderManual.setSubscriptionId(subscriptions.getId());
        ordersService.creatOrderManual(creatOrderManual);
        //创建订阅时间并且，发送订阅消息，并且更新用户模块的信息
        CreatSubscriptionManualDTO subscriptionManual = new CreatSubscriptionManualDTO();
        subscriptionManual.setProductId(orderByTradeNo.getProductId());
        subscriptionManual.setThirdSubscriptionId(thirdTradeNo);
        subscriptionManual.setStatus(SubscriptionConstant.actived);
        subscriptionManual.setUserId(subscriptions.getUserId());
        subscriptionManual.setAppId(subscriptions.getAppId());
        subscriptionManual.setEmail(userApi.getById(subscriptionManual.getUserId()).getResult().getEmail());
        //正式会员
        subscriptionManual.setSubscriptionType(1);
        subscriptionsService.creatSubscriptionManual(subscriptionManual);
    }
    void handleEmailSendBO (EmailCodeTypeEnum action, EmailSendBO bo) {
        //设置事件
        bo.setUseEvent(action.value());
        //设置title
        Map<String,String> titleMap = new HashMap<>();
        bo.setSendTitleContent(titleMap);
    }

    @Override
    public void handlingUserSubscriptionActivation(BigDecimal price, String thirdTradeNo, String thirdOrderNo, OrdersVO order, String thirdSubsId) {
        UserVO userVO = userApi.getByAppAccount(order.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
        if(ObjectUtils.isEmpty(userVO)){
            //创建账号
            userVO = register(order, userVO);
        }
        // 添加订阅信息
        String subId = CommonUtils.generateId();
        OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                .thirdTradeNo(thirdTradeNo)
                .price(price)
                .subscriptionId(subId)
                .thirdOrderNo(thirdOrderNo)
                .userId(userVO.getId())
                .build();
        // 修改订单状态
        OrderUpdateVO orderUpdate = ordersService.updateOrderSucceeded(orderSucceeded);
        CreatSubscriptionDTO creatSubscription = new CreatSubscriptionDTO();
        creatSubscription.setId(subId);
        creatSubscription.setUserId(orderUpdate.getUserId());
        creatSubscription.setOrderId(orderUpdate.getOrderId());
        creatSubscription.setAppId(orderUpdate.getAppId());
        creatSubscription.setPayment(order.getPayment());
        creatSubscription.setProductId(orderUpdate.getProductId());
        if(order.getPayment().equals(PaymentMethodEnum.PADDLE.getValue())){
            creatSubscription.setThirdSubscriptionId(thirdSubsId);
        }else{
            creatSubscription.setThirdSubscriptionId(thirdTradeNo);
        }
        creatSubscription.setEmail(userVO.getEmail());
        //如果是0则是试用
        creatSubscription.setSubscriptionType(2);
        subscriptionsService.creatSubscription(creatSubscription);
        // 用户信息
        // 查询订单信息获取产品id
        SubscriptionsVO subscriptionsVO = subscriptionsService.getSubscriptionBySubscriptionId(creatSubscription.getId());
        //发送邮件给用户
        EmailSendBO bo = new EmailSendBO();
        bo.setToEmail(userVO.getEmail());
        Map<String,String> contentMap = new HashMap<>();
        ProductVO productVO = productApi.getProduct(order.getProductId()).getResult();
        contentMap.put("@order_number@",order.getId());
        contentMap.put("@product@", productVO.getProductName());
        contentMap.put("@subscription@","yearly plan");
        DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm");
        contentMap.put("@purchase_date@",dateFormat.format(new Date()));
        contentMap.put("@price@",price.toString());
        contentMap.put("@exiration_date@",dateFormat.format(subscriptionsVO.getEndDate()));
        bo.setSendContent(contentMap);
        //设置内容 @order_number@ @product@ @subscription@ @purchase_date@  @price@ @exiration_date@
        handleEmailSendBO(EmailCodeTypeEnum.SUBSCRIPTION_SUCCESS,bo);
        emailApi.sendEmail(bo);
    }

    /**
     * 给未注册的购买用户创建账号
     * @param order order
     * @param userVO userVO
     * @return UserVO
     */
    private UserVO register(OrdersVO order, UserVO userVO) {
        UserRegisterDTO userRegisterDTO = new UserRegisterDTO();
        userRegisterDTO.setAppId(order.getAppId());
        String pwd = CommonUtils.generateRandomStringId(10);
        userRegisterDTO.setPassword(pwd);
        userRegisterDTO.setPlatformType(PlatformTypeEnum.WEBSITE.value().toString());
        userRegisterDTO.setUsername(order.getEmail());
        ProductVO productVO = productApi.getProduct(order.getProductId()).getResult();
        userRegisterDTO.setModel(productVO.getModel());
        ResultMap<Boolean> resultMap = userApi.register(userRegisterDTO);
        if(resultMap.getResult()){
            userVO = userApi.getByAppAccount(order.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
            //发送邮件给用户
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail(userVO.getEmail());
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@product_name@",productVO.getProductName());
            contentMap.put("@plan_name@"," yearly plan");
            contentMap.put("@account@", userVO.getEmail());
            contentMap.put("@password@",pwd);
            bo.setSendContent(contentMap);
            //设置内容 @product_name@ @plan_name@ @account@ @password@
            handleEmailSendBO(EmailCodeTypeEnum.SUBSCRIPTION_SUCCESS_WITH_REGISTER,bo);
            emailApi.sendEmail(bo);
        }
        return userVO;
    }


    /**
     * 取消自动订阅的后续操作
     * @param thirdSubscriptionId 第三方订阅id
     */
    @Override
    public void handleUnsubscribeFollowUpActions(String thirdSubscriptionId) {
        SubscriptionsVO subscription = subscriptionsService.getSubscriptionByThirdSubscriptionId(thirdSubscriptionId);
        String userId = subscription.getUserId();
        String productId = subscription.getProductId();
        // 根据用户id和产品id修改用户订阅状态 不是自动订阅
        UserSubscriptionInfo userSubscriptionInfo = userSubscriptionInfoApi.getByUserIdAndProductId(userId, productId).getResult();
        UserSubscriptionInfo info = new UserSubscriptionInfo();
        info.setUserId(userId);
        info.setProductId(productId);
        info.setPayType(PDFOfficeUserSubscriptionPayTypeEnum.NON_AUTO_SUBSCRIPTION.value());
        info.setId(userSubscriptionInfo.getId());
        userSubscriptionInfoApi.update(info);
    }
}
