package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CancelSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


@RestController
@RequestMapping("/pay")
@RequiredArgsConstructor
public class PayController {

    private final PaymentService paymentService;

    private final ProductInfoService productInfoService;

    /**
     * 创建订单
     *
     * @param  creatOrderDTO  creatOrderDTO
     * @return ResultMap
     */
    @PostMapping("/creatOrder")
    public ResultMap<String> creatOrder(@RequestBody CreatOrderDTO creatOrderDTO) {
        return ResultMap.success(paymentService.creatOrder(creatOrderDTO));
    }

    /**
     * 订单创建（applePay专用）
     *
     * @param creatOrderDTO creatOrderDTO
     * @return CreatOrderResultVO
     */
    @PostMapping("/appStoreCreateOrder")
    public ResultMap<CreatOrderResultVO> appStoreCreateOrder(@RequestBody CreatOrderDTO creatOrderDTO){
        return ResultMap.success(paymentService.appStoreCreateOrder(creatOrderDTO));
    }

    /**
     * 订单支付成功（applePay专用）
     * 回传appal pay id &签名证书
     *
     * @param appStoreOrderSucceed appStoreOrderSucceed
     * @return Void
     */
    @PostMapping("/appStoreOrderSucceed")
    public ResultMap<Void> appStoreOrderSucceed(@Validated @RequestBody AppStoreOrderSucceedDTO appStoreOrderSucceed){
        paymentService.appStoreOrderSucceed(appStoreOrderSucceed);
        return ResultMap.success();
    }

    /**
     * 取消订阅
     * @param productId productId
     * @return String
     */
    @PostMapping("/cancelSubscription")
    public ResultMap<String> cancelSubscription(@RequestParam String productId) {
        paymentService.cancelSubscription(productId);
        return ResultMap.success(CommonConstant.RESULT_SUCCESS);
    }
}
