package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CancelSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreatOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.SubscriptionApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreatOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PlatformTypeEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.dto.IsSubscriptionDTO;
import cn.kdan.cloud.pdf.office.website.service.PaymentService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class PaymentServiceImpl implements PaymentService {

    private final SubscriptionApi subscriptionApi;

    private final OrderApi orderApi;

    private final UserApi userApi;

    @Override
    public ResultMap<SubscriptionSchemeVO> getSubscriptionSchemeInfo(String appId) {
        String userId = SecurityUtils.getUserId();
        return subscriptionApi.getSubscriptionSchemeInfo(userId,appId);
    }

    @Override
    public LocalDateTime getSubscriptionEndDate(Integer platform) {
        String userId = SecurityUtils.getUserId();
        return MyDateUtils.dateToLocalDateTime(subscriptionApi.getSubscriptions(userId, platform).getResult().getEndDate());
    }

    @Override
    public String creatOrder(CreatOrderDTO creatOrderDTO) {
        UserVO userVO = userApi.getByAppAccount(creatOrderDTO.getEmail(), PlatformTypeEnum.WEBSITE.value().toString()).getResult();
        if(!ObjectUtils.isEmpty(userVO.getId())){
            // 查询订阅信息，有返回false，无返回true
            List<SubscriptionsVO> subscriptionsVOS = subscriptionApi.getSubscriptionsByUserIdAndProductId(userVO.getId(),creatOrderDTO.getProductId()).getResult();
            if (CollectionUtils.isEmpty(subscriptionsVOS) || subscriptionsVOS.size() == 0){
                //1折扣0不折扣
                creatOrderDTO.setIsDiscount(1);
            }else{
                creatOrderDTO.setIsDiscount(0);
            }
            creatOrderDTO.setUserId(userVO.getId());
        }else {
            creatOrderDTO.setIsDiscount(1);
        }
        return orderApi.creatOrder(creatOrderDTO).getResult().getPagePayUrl();
    }

    @Override
    public void cancelSubscription(String productId) {
        String userId = SecurityUtils.getUserId();
        //获取用户对于某个产品订阅列表
        List<SubscriptionsVO> subscriptionsVOS = subscriptionApi.getSubscriptionsByUserIdAndProductId(userId,productId).getResult();
        List<SubscriptionsVO> activeResult = subscriptionsVOS.stream()
                .filter(subscription -> subscription.getStatus() == 3)
                .collect(Collectors.toList());
        //用户对于某个产品活跃的订阅只有一个，根据订阅id去取消续订
        CancelSubscriptionDTO dto = new CancelSubscriptionDTO();
        if(!CollectionUtils.isEmpty(activeResult)){
            dto.setSubscriptionId(activeResult.get(0).getId());
            orderApi.cancelSubscription(dto);
        }
    }

    @Override
    public CreatOrderResultVO appStoreCreateOrder(CreatOrderDTO creatOrderDTO) {
        return orderApi.appStoreCreateOrder(creatOrderDTO).getResult();
    }

    @Override
    public void appStoreOrderSucceed(AppStoreOrderSucceedDTO appStoreOrderSucceed){
        orderApi.appStoreOrderSucceed(appStoreOrderSucceed);
    }

    @Override
    public boolean isSubscription(IsSubscriptionDTO isSubscriptionDTO) {
        // 判断用户是否存在
        UserVO userVO = userApi.getByEmail(isSubscriptionDTO.getEmail()).getResult();
        if (ObjectUtils.isEmpty(userVO)) {
            return true;
        }
        if (!PDFOfficeUserStatusEnum.VALID.value().toString().equals(userVO.getValidFlag())){
            throw new BackendRuntimeException(ExceptionEnum.ABNORMAL_USER_STATUS);
        }
        // 查询订阅信息，有返回false，无返回true
        List<SubscriptionsVO> subscriptionsVOS = subscriptionApi.getSubscriptionsByUserIdAndProductId(userVO.getId(),isSubscriptionDTO.getProductId()).getResult();
        if (CollectionUtils.isEmpty(subscriptionsVOS) || subscriptionsVOS.size()<=0){
            return true;
        }
        return false;
    }


}
